// sqlauth.c
#include <rusproxy.h>
#include <string.h>
#include <libpq-fe.h>
#include <syslog.h>

static PGconn  *conn;
static const char *conninfo = "dbname=rusproxy user=rusproxy password=mirabeth host=node0.k1.com.br";
static const char *pgsql_name = "auth_user";

// Function to authenticate using PostgreSQL
static int
authenticate_with_postgres(const char *email, const char *password)
{
	PGresult       *res;
	char	       *result_login;
	int		ret_code = 0;	// 0 represents failure, 1 represents success
	const char     *paramValues [2] = {email, password};
	int		paramLengths [2] = {0, 0};	// Length can be 0 for text format
	int		paramFormats [2] = {0, 0};	// 0 = text format

	res = PQexecPrepared (conn, pgsql_name, 2, paramValues, paramLengths, paramFormats, 0);
	if (PQresultStatus (res) != PGRES_TUPLES_OK)
	{
		syslog (LOG_ERR, "Authentication query failed: %s", PQerrorMessage (conn));
		PQclear (res);
		return FALSE;	// Failure
	}

	if (PQntuples (res))
	{
		result_login = PQgetvalue (res, 0, 0);
		fprintf (stderr, "email:%s, login:%s\n", email, result_login);
		return TRUE;
	}

	// Check if the user was found
	if (PQntuples (res) == 0)
		PQclear (res);
	return ret_code;
}

// Main function for testing the authentication logic
int
main()
{
	char		input_buffer [1024];
	gchar	      **tokens = NULL;
	PGresult       *res;

	// Initialize syslog for logging
	openlog ("sqlauth", LOG_PID | LOG_CONS, LOG_AUTH);

	// Connect to the PostgreSQL database
	conn = PQconnectdb (conninfo);
	if (PQstatus (conn) != CONNECTION_OK)
	{
		syslog (LOG_ERR, "PostgreSQL connection failed: %s", PQerrorMessage (conn));
		PQfinish (conn);
		closelog ();
		return 1;	// Failure
	}

	res = PQprepare (conn, pgsql_name, "SELECT login,password FROM proxy_users WHERE email = $1 AND password = $2 AND balance > 0", 2, NULL);
	if (PQresultStatus (res) != PGRES_COMMAND_OK)
	{
		syslog (LOG_ERR, "Statement preparation failed: %s", PQerrorMessage (conn));
		PQclear (res);
		PQfinish (conn);
		return 1;	// Failure
	}
	PQclear (res);
	// Initialize the prepared statement

	// Read the input from stdin in the format "email password"
	while (fgets (input_buffer, sizeof (input_buffer), stdin))
	{
		tokens = g_strsplit_set (g_strstrip (input_buffer), " \t", 2);
		if (tokens == NULL || g_strv_length (tokens) != 2)
		{
			syslog (LOG_ERR, "Invalid input format. Please provide 'user password'");
			g_strfreev (tokens);
			continue;	// Skip to next iteration
		}

		// Perform authentication
		if (authenticate_with_postgres (tokens [0], tokens [1]))
		{
			syslog (LOG_INFO, "Authentication successful for user: %s", tokens [0]);
			printf ("OK\n");	// Authentication successful
		} else
		{
			syslog (LOG_ERR, "Authentication failed for user: %s", tokens [0]);
			printf ("ERR\n");	// Authentication failed
		}
		g_strfreev (tokens);
	}

	// Clean up
	PQfinish (conn);
	closelog ();

	return 0;
}
